<!---
/* *************************************************************** */
Author: 	
	PaperThin Inc.
	OHSU
ADF App:
	OHSU Nav
Resources:
	OHSUNavCSS must be registered as a name or alias of a style sheet resource in CommonSpot 10 as
			this script loads the style sheet based on that name.
History:
	2009-05-28 - RLW - Created
	2013-02-05 - MFC - Added check to see if the page is in the navigation.
	2015-09-30 - DMB - Converted to CFC for CommonSpot 10.
	2016-02-29 - DRM - Resource detection support
							 Removed requirement to register app stylesheet as a resource, loading it unregistered instead
	2016-05-06 - GAC - Updated to include the registered resource so it will be included in the exported CFT .zip archive
--->
<cfcomponent displayName="OHSUNavRender" extends="ADF.extensions.customfields.adf-form-field-renderer-base">

<cffunction name="renderControl" returntype="void" access="public">
	<cfargument name="fieldName" type="string" required="yes">
	<cfargument name="fieldDomID" type="string" required="yes">
	<cfargument name="value" type="string" required="yes">
	
	<cfscript>
		var inputParameters = Server.CommonSpot.UDF.util.duplicateBean(arguments.parameters);
		var currentValue = arguments.value;	// the field's current value
		var readOnly = (arguments.displayMode EQ 'readonly') ? true : false;
		var initArgs = StructNew();
		var selectionArgs = StructNew();
		var canDoNav = 1;
		var pageID = 0;
		var pageIsStartPoint = 0;
		var pageIsPromoted = 0;
		var pageName = "doc.cfm";
		var pageTitle = "";
		var fqFieldName = arguments.fieldName;

		// if this is a new page/document [prevents objects like flash/video/audio ]
		if( structKeyExists(request.params, "dialogDescriptor") and request.params.dialogDescriptor eq "createPage" )
		{
			// this is a page and we know the name (which will get passed in to check if it can be a startPoint)
			if( structKeyExists(request.params, "name"))
				pageName = "#request.params.name#.cfm";
			pageTitle = "#request.params.title#";
			// note the pageID will be retrieved from the hidden field "savePageID" via JS
			subsiteID = request.subsite.ID;
		}
		// if this is a page that already exists [page prop on page/doc/external]
		else if( structKeyExists(request.params, "pageID") and request.params.pageID gt 2 )
		{
			// retrieve this pages standard metadata
			pageMetadata = application.ohsuNav.csData.getStandardMetadata(request.params.pageID);
			pageName = pageMetadata.fileName;
			// use form parameters unless they don't exist
			if( structKeyExists(request.params, "title") and len(request.params.title) )
				pageTitle = request.params.title;
			else
				pageTitle = pageMetadata.title;
			pageID = request.params.pageID;
			pageIsStartPoint = application.ohsuNav.navElementService.isStartPoint(pageID);
			pageIsPromoted = application.ohsuNav.navElementService.isPromotedToParent(pageID);
			subsiteID = pageMetadata.subsiteID;
		}
		// otherwise no nav
		else
			canDoNav = 0;
	</cfscript>

	<cfif canDoNav>
		<cfscript>
			subsiteLinks = application.ohsuNav.navElementService.getSubsiteLinks(subsiteID);
		</cfscript>

		<cfoutput>
			<script type="text/javascript">	
				// javascript validation to make sure they have text to be converted
				#fqFieldName#=new Object();
				#fqFieldName#.id='#arguments.fieldName#';
				#fqFieldName#.tid=#arguments.renderTabIndex#;
				#fqFieldName#.validator="#fqFieldName#updateNav()";
				//#fqFieldName#.msg="Please upload a document.";
				// push on to validation array
				vobjects_#arguments.formName#.push(#fqFieldName#);
	
				var hasSubsiteLinks = <cfif len(subsiteLinks.uniqueID)>true<cfelse>false</cfif>;
				var uniqueID = '#subsiteLinks.uniqueID#';
				var startPointAllowed = false;
				var promoteToParentAllowed = false;
				var checkedActions = false;
				var pageIsStartPoint = #pageIsStartPoint#;
				var pageIsPromoted = #pageIsPromoted#;
				// start up			
				jQuery( function() {
					pageID = #pageID#;
					// set the PageID via jQuery - to handle savePageID for new pages
					if( jQuery('input[name="savePageID"]') ){
					
						pageID = jQuery('input[name="savePageID"]').attr("value");
						// reset the pageID for the new page link
						jQuery('.newLink').attr("id", pageID);
					}
					jQuery("##newPage").sortable({
						connectWith: '.subsiteLinks',
						scroll: true
					});
					jQuery(".subsiteLinks").sortable({
						connectWith: '.trash',
						scroll: true
					});
					jQuery(".trash").sortable({
						receive: function(event, ui){
							thisItem = ui.item.children().html();
							if( confirm("Are you sure you want to remove '" + thisItem + "' from the list" ) )
								;//ui.item.remove();
							else
								ui.item.appendTo('##subsiteLinks');
						}
					})
					// bind addToNav
					jQuery("##addToNav").bind('click', handleActionDisplay);
					
					// handle styles for the links
					jQuery(".links").addClass('ui-state-default');
					jQuery(".links").mouseover( function(){
						jQuery(this).addClass('ui-state-hover highlighted');
					}).mouseout( function(){
						jQuery(this).removeClass('ui-state-hover highlighted');
					});
					// make the new link active
					jQuery(".newLink").addClass('ui-state-active')

					// default all actions to off
					overrideActions();
				
				});
				function pageIncluded(){
					var pageIsIncluded = false;
					var found = jQuery('.subsiteLinks').find("##" + pageID);
					// if the page is in the subsiteLinks already or its the startpoint
					if( found.length > 0 || pageIsStartPoint )
						pageIsIncluded = true;
					return pageIsIncluded;
				}

				// handle how the checkboxes are displayed based on business logic and actions
				function handleActionDisplay(){
				
					if( jQuery("##addToNav").attr("checked") == false ){
						// move the link back to the holding area
						jQuery('##subsiteLinks').find(".newLink").remove().appendTo('##newPage');
						// reset the options
						overrideActions();
					} else {
						// move the link over into the bottom of the list
						jQuery('##newPage').find(".newLink").remove().appendTo('##subsiteLinks');				
						// if we haven't checked business logic yet for startPoints or promoteToParent
						if( !checkedActions )
							checkActions();
						// can we make this page a startPoint
						if( startPointAllowed ){
							jQuery("##makeStartPoint").attr("disabled", false);
						}
						// can we promote to parent
						if( promoteToParentAllowed ){
							jQuery("##promoteToParent").attr("disabled", false);				
						}
						// if there are no links yet - must have start point
						if( !hasSubsiteLinks ){
							jQuery("##makeStartPoint").attr("disabled", true);
							jQuery("##makeStartPoint").attr("checked", true);
							jQuery("##promoteToParent").attr("disabled", true);
							jQuery("##promoteToParent").attr("checked", true);
						}
					}		
				}
				// turn off/default
				function overrideActions(){
				
					jQuery("##addToNav").attr("checked", false);
					jQuery("##makeStartPoint").attr("disabled", true);
					jQuery("##makeStartPoint").attr("checked", false);
					jQuery("##promoteToParent").attr("disabled", true);
					jQuery("##promoteToParent").attr("checked", false);
				
					//if this page is in the nav already this will be checked and disabled
					if( pageIncluded() ) {
						jQuery("##addToNav").attr("disabled", true);
						jQuery("##addToNav").attr("checked", true);
						// if the page is included in both areas remove it from the start point
						jQuery('##newPage').find(".newLink").remove();
					}			
				}
				// check the business logic
				function checkActions(){
					// if we have not checked actions yet do that
					if( !checkedActions){
						// check for startPoint
						jQuery.ajax({
							async: false,
							url: "#application.ADF.ajaxProxy#",
							data: { bean: "navElementService",
								method: "startPointAllowed",
								fileName: "#pageName#",
								subsiteID: #subsiteID# },
							success: function(isAllowed){
								if(isAllowed == "true") startPointAllowed = true; }
						});
						// check for promoteToParent
						jQuery.ajax({
							async: false,
							url: "#application.ADF.ajaxProxy#",
							data: { bean: "navElementService", 
								method: "promoteToParentAllowed",
								fileName: "#pageName#",
								subsiteID: #subsiteID# },
							success: function(isAllowed){
								if(isAllowed == "true") promoteToParentAllowed = true; }
						});
						checkedActions = true;	
					}
				}
				// Handle on submit
				function #fqFieldName#updateNav(){
				
					// Check if the page is in the navigation
					if ( jQuery("##addToNav").is(":checked") == "true" )
						updateNavRecord = true;
					else
						updateNavRecord = true;
				
					if ( updateNavRecord ){
				
						startPoint = false;
						promote = false;
						// set the childList from the list of available pages
						// 2010-08-20 - MFC - Updated to get the Array out then create a string list
						childArray = jQuery("##subsiteLinks").sortable('toArray');
						childList = "";
						// Loop over the array to set the list values
						jQuery.each(childArray, function(index, value) {
							// Validate if the item has value
							if ( value != '' && value > 0 ){
								if ( childList.length <= 0 )
									childList = value;
								else
									childList = childList + "," + value;
							}
						});
					
						if( childList.length <= 0 )
							childList = "";
						// check the status of startPoint and promoteToParent
						if( jQuery("##makeStartPoint").attr("checked") )
							startPoint = true;
						// if this is promoted we don't care about the child list so clear it
						if( jQuery("##promoteToParent").attr("checked") )
						{
							promote = true;
							// clear the child list for a new parent
							childList = "";	
						}
						// are we doing a new startPoint
						if( startPoint ){
						
							jQuery.ajax({
								async: false,
								url: "#application.ADF.ajaxProxy#",
								type: "POST",
								data: { bean: "navElementController",
									method: "addStartPoint",
									nCurrentPageID: pageID,
									sParentChildList: childList,
									sParentID: uniqueID,
									nSubsiteID: #subsiteID#,
									bIsPromoted: promote },
								success: function(msg){
									//alert("new start point success");
									return true;
								},
								error: function(msg){
									//alert("new start point error");
									return false;
								}
							});		
						} else {
						
							jQuery.ajax({
								async: false,
								url: "#application.ADF.ajaxProxy#",
								type: "POST",
								data: {
									bean: "navElementController",
									method: "updateChildList",
									uniqueID: uniqueID,
									childList: childList },
								success: function(){
									//alert("update success");
								},
								error: function(msg){
									//alert("update error");
									// TODO: should kick off an error log
								}
							});							
						}
					}
					return true;
				}
			</script>
			
			<div style="" id="main">
			  <div id="col1">
			    <ul style="" id="newPage" class="inactive ui-sortable">
					<li class="newLink" id=""><div class="itemText">#pageTitle#</div></li>
			    </ul>
			    <input id="addToNav" type="checkbox">
			    <label for="addToNav">Add to Navigation</label>
			    <br>
			    <input id="makeStartPoint" type="checkbox">
			    <label for="makeStartPoint">Create Start Point?</label>
			    <br>
			    <input id="promoteToParent" type="checkbox">
			    <label for="promoteToParent">Promote to Parent?</label>
			    <br>
			<!--- Trash --->    
			    <div id="trash"> <img src="/ADF/apps/ohsu_nav/images/DD_trash.png">
			      <ul style="" class="trash">
			        <li style="" class="links">and me</li>
			      </ul>
			    </div>
			  </div>
			<!--- End Col 1 & Start Col 2 --->  
			  <div id="col2">
				<cfif subsiteLinks.navIsCurrentSubsite>
					<h3 class="subsiteHeader">#request.subsite.menuName# Links (in order)</h3>
				<!--- // this is the parents subsite navigation --->
				<cfelseif len(subsiteLinks.uniqueID)>
					<h3 class="subsiteHeader">#request.subsiteCache[request.subsiteCache[subsiteID].parentID].menuName# Links(in order)</h3>
				<!--- // there is no navigation yet --->
				<cfelse>
					<h3 class="subsiteHeader">No navigation exists for this subsite or parent</h3>
				</cfif>
			    <div id="inner">
			      <ul style="" class="subsiteLinks" id="subsiteLinks">
					<cfloop from="1" to="#arrayLen(subsiteLinks.pageArray)#" index="itm">
						<!--- // if this is the current page add the new link to it --->
						<cfif subsiteLinks.pageArray[itm].pageID eq pageID>
							<cfset class = "newLink">
							<script type="text/javascript">
								// remove the link from the holding bin
								jQuery('##newPage').find(".newLink").remove();
							</script>
						<cfelse>
							<cfset class = "links">
						</cfif>
						<li class="#class#" id="#subsiteLinks.pageArray[itm].pageID#"><div class="itemText">#subsiteLinks.pageArray[itm].pageTitle#</div></li>
					</cfloop>
			      </ul>
			    </div>
			  </div>
			</div>
		</cfoutput>
	</cfif>
</cffunction>

<cfscript>
	private string function isMultiline()
	{
		return '1';  // Tell CommonSpot it's a tall dialog.
	}

	public string function getResourceDependencies()
	{
		return listAppend(super.getResourceDependencies(), "jQuery,jQueryUI,OHSUNavCSS");
	}
	public string function loadResourceDependencies()
	{
		application.ohsuNav.scripts.loadResources("JQuery,JQueryUI,OHSUNavCSS");
		
		// Use as registered respources so the definition is included in the .zip export package
		//loadUnregisteredResource("/ADF/apps/ohsu_nav/style/interface.css", "StyleSheet", "head", "secondary", 1, 1);
	}
</cfscript>
</cfcomponent>